@@ -296,57 +296,92 @@ lookup_stock_compat (const char *id)
 	return NULL;
 }
 
-GdkPixbuf *
-bonobo_ui_util_xml_get_pixbuf (GtkWidget    *widget,
-			       BonoboUINode *node,
-			       GtkIconSize   icon_size)
+static GHashTable *pixbuf_cache = NULL;
+
+void
+bonobo_ui_image_cache_trash (void)
+{
+	if (pixbuf_cache)
+		g_hash_table_destroy (pixbuf_cache);
+	pixbuf_cache = NULL;
+}
+
+void
+bonobo_ui_image_set_pixbuf (GtkImage *image, GdkPixbuf *pixbuf)
+{
+	if (gtk_image_get_pixbuf (image) != pixbuf)
+		gtk_image_set_from_pixbuf (image, pixbuf);
+
+	else if (pixbuf)
+		g_object_unref (pixbuf);
+}
+
+static void
+bonobo_ui_image_set_stock (GtkImage   *image,
+			   const char *name,
+			   GtkIconSize icon_size)
+{
+	g_return_if_fail (name != NULL);
+
+	if (image->storage_type != GTK_IMAGE_STOCK ||
+	    image->icon_size != icon_size ||
+	    !image->data.stock.stock_id ||
+	    strcmp (image->data.stock.stock_id, name))
+		gtk_image_set_from_stock (image, name, icon_size);
+}
+
+void
+bonobo_ui_util_xml_set_image (GtkImage     *image,
+			      BonoboUINode *node,
+			      BonoboUINode *cmd_node,
+			      GtkIconSize   icon_size)
 {
-	GdkPixbuf  *pixbuf = NULL;
 	char       *key;
 	const char *type, *text;
-	/* FIXME: this cache needs to be invalidated on theme
-	 * changes */
-	static GHashTable *pixbuf_cache = NULL;
+	GdkPixbuf  *pixbuf = NULL;
 
-	g_return_val_if_fail (node != NULL, NULL);
+	g_return_if_fail (node != NULL);
 
-	if (!(type = bonobo_ui_node_peek_attr (node, "pixtype")))
-		return NULL;
+	if (!(type = bonobo_ui_node_peek_attr (node, "pixtype")) && cmd_node &&
+	    !(type = bonobo_ui_node_peek_attr (cmd_node, "pixtype")))
+		return;
 
-	if (!(text = bonobo_ui_node_peek_attr (node, "pixname")))
-		return NULL;
+	if (!(text = bonobo_ui_node_peek_attr (node, "pixname")) && cmd_node &&
+	    !(text = bonobo_ui_node_peek_attr (cmd_node, "pixname")))
+		return;
 
-	key = g_strdup_printf ("%s:%s:%d:%d", type, text,
-			       icon_size,
-			       gtk_widget_get_direction (widget));
+	if (!strcmp (type, "stock")) {
+		if (gtk_icon_factory_lookup_default (text))
+			bonobo_ui_image_set_stock (image, text, icon_size);
+		else {
+			char *mapped;
+			if ((mapped = lookup_stock_compat (text))) {
+				bonobo_ui_image_set_stock (image, mapped, icon_size);
+				g_free (mapped);
+			}
+		}
+		return;
+	}
+
+	/* FIXME: how worthwhile is this cache */
+	key = g_strdup_printf (
+		"%s:%d:%d", text, icon_size,
+		gtk_widget_get_direction (GTK_WIDGET (image)));
 
 	if (!pixbuf_cache)
-		pixbuf_cache = g_hash_table_new (g_str_hash, g_str_equal);
+		pixbuf_cache = g_hash_table_new_full (
+			g_str_hash, g_str_equal,
+			(GDestroyNotify) g_free,
+			(GDestroyNotify) g_object_unref);
 
 	else if ((pixbuf = g_hash_table_lookup (pixbuf_cache, key))) {
 		g_free (key);
 		g_object_ref (pixbuf);
-		return pixbuf;
+		bonobo_ui_image_set_pixbuf (image, pixbuf);
+		return;
 	}
 
-	if (!strcmp (type, "stock")) {
-		if (gtk_icon_factory_lookup_default (text))
-			pixbuf = gtk_widget_render_icon (
-				widget, text, icon_size, "bonobo-ui");
-		else {
-			char *mapped;
-
-			if ((mapped = lookup_stock_compat (text))) {
-				
-				pixbuf = gtk_widget_render_icon (
-					widget, mapped, icon_size, "bonobo-ui");
-
-				g_free (mapped);
-			} else
-				g_warning ("Unknown stock icon '%s', stock names all changed in Gtk+ 2.0", text);
-		}
-
-	} else if (!strcmp (type, "filename")) {
+	if (!strcmp (type, "filename")) {
 		char *name = find_pixmap_in_path (text);
 
 		if ((name == NULL) || !g_file_test (name, G_FILE_TEST_EXISTS))
@@ -368,8 +403,7 @@ bonobo_ui_util_xml_get_pixbuf (GtkWidget    *widget,
 	} else
 		g_free (key);
 
-
-	return pixbuf;
+	bonobo_ui_image_set_pixbuf (image, pixbuf);
 }
 
 /**
@@ -442,13 +476,6 @@ bonobo_ui_util_xml_get_icon_widget (BonoboUINode *node, GtkIconSize icon_size)
 	return image;
 }
 
-void
-bonobo_ui_image_set_pixbuf (GtkImage *image, GdkPixbuf *pixbuf)
-{
-	if (gtk_image_get_pixbuf (image) != pixbuf)
-		gtk_image_set_from_pixbuf (image, pixbuf);
-}
-
 /**
  * bonobo_ui_util_xml_set_pixbuf:
  * @node: the node
@@ -569,7 +596,7 @@ bonobo_ui_util_build_help_menu (BonoboUIComponent *listener,
 				BonoboUINode      *parent)
 {
 	static int unique = 0;
-	char *id, *help_path;
+	char *id;
 	BonoboUINode *node;
 	HelpDisplayClosure *cl;
  
@@ -596,6 +623,8 @@ bonobo_ui_util_build_help_menu (BonoboUIComponent *listener,
 			cl, help_display_closure_free));
 
 	bonobo_ui_node_add_child (parent, node);
+
+	g_free (id);
 }
 
 /**
